       MODULE module_mp_nudge_light

!      lightning assimilation subroutine - Fierro et al. (2012, MWR)

#ifdef DM_PARALLEL
         USE module_dm, ONLY : &
            local_communicator, mytask, ntasks, ntasks_x, ntasks_y,mytask_x, mytask_y
#endif

        CONTAINS

         SUBROUTINE nudge_light(nx,ny,nz,qg,qv,gridlight,temp,press,dz,dx,dy,dt)

          implicit none
#ifdef DM_PARALLEL
        INCLUDE 'mpif.h'
#endif

         real ::  RH= 0.95 ! minimum RH for nudging: larger RH -> larger grid volume activated for nudging
         real ::  QG_TRESH=0.003 ! threshold nudging on graupel mass to further constraint nudging volume.
         real ::  t_bot=278.15,zerok=273.15 !top of nudging layer
         real ::  t_up= 253.15 ! bottom of nudging layer
         real ::  MAX_RH= 1.02 
         real ::  BB= 0.25 ! controls the slope of TANH curve
         real ::  CC= 0.25 ! controls spacing of QG curves in eq 1 o
         real ::  LVo=2.501E+06 ! Latent heat of vaporization - assume const
         real ::  Rv=461.5
         real ::  To=273.15
         real ::  ep1=0.6222
         real ::  ep2=611.73 ! in Pa
         real ::  const=2.2 ! do not change

         real AA ! controls how much % is increased above lowest threshold (set by RH) 

          integer i,j,k,nx,ny,nz
          real  gridlight(nx,ny)
          real qg(nx,nz,ny)
          real qv(nx,nz,ny)
          real temp(nx,nz,ny) ! in deg Kelvin
          real press(nx,nz,ny) ! in Pa
          real qvsat,qvsat_r ! in kg/kg
          real esw ! in Pa
          real angle
          real nudge_coeff

          real, dimension(nx,nz,ny), intent(in):: dz
          real, intent(IN) :: dx,dy,dt
          
          double precision :: dv, total_vapor, tmp, total_vapor_out
          integer mpi_error_code

         if (maxval(gridlight).gt.0.) then
         write(0,*) 'MAXVAL GRIDLIGHT',maxval(gridlight)
!         write(0,*) 'MAXVAL ARRAYS',maxval(temp),maxval(press),maxval(qg),maxval(qv)
!         write(0,*) 'dx,dy,dz',dx,dy,maxval(dz)
         endif

         AA = MAX_RH - RH
        
        nudge_coeff = 1/dt ! direct insertion as in Fierro et al. (2012).

        total_vapor = 0.
        
        do i=1,nx
           do k=1,nz
              do j=1,ny
              

              dv = dx*dy*dz(i,k,j)

!          for QVS: Clausius Clapeyron Equation (constant Latent Heat of
!          Vaporization Lv) - Taken from Rogers and Yau (1989)

           qvsat=(ep1/press(i,k,j))*(ep2*exp( (LVo/Rv)*((1./To)-(1./(temp(i,k,j)) ) ) ) )

           if(temp(i,k,j) .gt. zerok) then
           qvsat_r=rslf(press(i,k,j),temp(i,k,j))
           else
           qvsat_r=rsif(press(i,k,j),temp(i,k,j))
           endif

           angle=(1000.*qg(i,k,j))**const

          if (gridlight(i,j).ge.1.) then
          if (qg(i,k,j).lt.QG_TRESH) then
          if (qv(i,k,j).lt.RH*qvsat) then 
            if (temp(i,k,j).lt.t_bot.and.temp(i,k,j).gt.t_up) then 

            tmp = RH*qvsat+AA*qvsat*tanh(BB*gridlight(i,j))*(1.-tanh(CC*angle))
            
            total_vapor = total_vapor + Min(1.,nudge_coeff*dt)*(tmp-qv(i,k,j))*dv

            qv(i,k,j) = qv(i,k,j) + Min(1.,nudge_coeff*dt)*(tmp -qv(i,k,j))
 
            endif
          endif
          endif
          endif

              enddo
            enddo
          enddo

#ifdef DM_PARALLEL
      CALL MPI_Reduce(total_vapor, total_vapor_out, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, local_communicator, mpi_error_code)

      
      
      IF ( mytask == 0 ) THEN
         total_vapor = total_vapor_out
#endif
       write(0,*) 'Total vapor mass added: ',total_vapor
       
#ifdef DM_PARALLEL
       ENDIF
#endif
       
       
        RETURN
        END SUBROUTINE nudge_light

         SUBROUTINE r3dvar(nx,ny,nz,qg,qv,rv_3d,temp,press,dz,dx,dy,dt)

          implicit none
#ifdef DM_PARALLEL
        INCLUDE 'mpif.h'
#endif

         real ::  RH= 0.95 ! minimum RH for nudging: larger RH -> larger grid volume activated for nudging
         real ::  QG_TRESH=0.003 ! threshold nudging on graupel mass to further constraint nudging volume.
         real ::  RH_TRESH=90.
         real ::  t_bot=273.15, zerok=273.15 !top of nudging layer
         real ::  t_up= 253.15 ! bottom of nudging layer
         real ::  MAX_RH= 1.02 
         real ::  BB= 0.25 ! controls the slope of TANH curve
         real ::  CC= 0.25 ! controls spacing of QG curves in eq 1 o
         real ::  LVo=2.501E+06 ! Latent heat of vaporization - assume const
         real ::  Rv=461.5
         real ::  To=273.15
         real ::  ep1=0.6222
         real ::  ep2=611.73 ! in Pa
         real ::  const=2.2 ! do not change
         real, parameter :: amiss=-999.

         real AA ! controls how much % is increased above lowest threshold (set by RH) 

          integer i,j,k,nx,ny,nz,ic_tot
          real  gridlight(nx,ny)
          real :: rv_3d(nx,nz,ny)
          real qg(nx,nz,ny)
          real qv(nx,nz,ny)
          real temp(nx,nz,ny) ! in deg Kelvin
          real press(nx,nz,ny) ! in Pa
          real qvsat,qvsat_r ! in kg/kg
          real esw ! in Pa
          real angle
          real nudge_coeff,rh_wrf

          real, dimension(nx,nz,ny), intent(in):: dz
          real, intent(IN) :: dx,dy,dt
          
          double precision :: dv, total_vapor, tmp, total_vapor_out
          integer mpi_error_code

!          print*,'Sono nella routine...3dvar'

!         if (maxval(gridlight).gt.0.) then
!         write(0,*) 'MAXVAL GRIDLIGHT',maxval(gridlight)
!         write(0,*) 'MAXVAL ARRAYS',maxval(temp),maxval(press),maxval(qg),maxval(qv)
!         write(0,*) 'dx,dy,dz',dx,dy,maxval(dz)
!         endif

         AA = MAX_RH - RH
        
        nudge_coeff = 1/(1.5*dt) ! direct insertion as in Fierro et al. (2012).

        total_vapor = 0.
      
        ic_tot=0 
        do i=1,nx
           do k=1,nz
              do j=1,ny
              if(rv_3d(i,k,j) .ne. amiss) then 
              ic_tot=ic_tot+1
              endif     
              enddo
           enddo
        enddo

        if(ic_tot .ge. 1) then
        print*,'IC_TOT ------ ',ic_tot
        end if 
 
        do i=1,nx
           do k=1,nz
              do j=1,ny
              

              dv = dx*dy*dz(i,k,j)

!          for QVS: Clausius Clapeyron Equation (constant Latent Heat of
!          Vaporization Lv) - Taken from Rogers and Yau (1989)

           qvsat=(ep1/press(i,k,j))*(ep2*exp( (LVo/Rv)*((1./To)-(1./(temp(i,k,j)) ) ) ) )
           if(temp(i,k,j) .gt. zerok) then
           qvsat_r=rslf(press(i,k,j),temp(i,k,j))
           else
           qvsat_r=rsif(press(i,k,j),temp(i,k,j))
           endif
          
           rh_wrf=100.*qv(i,k,j)/qvsat_r

!           print*,i,j,k,press(i,k,j),temp(i,k,j),qvsat,qvsat_r

           angle=(1000.*qg(i,k,j))**const

          if (rv_3d(i,k,j) .ne. amiss) then
          if (rh_wrf .lt. RH_TRESH) then
!          if (qv(i,k,j).lt.RH*qvsat) then 
!            if (temp(i,k,j).lt.t_bot.and.temp(i,k,j).gt.t_up) then 

            if(rv_3d(i,k,j) .gt. 100.) rv_3d(i,k,j)=100.
!!!  ATTENZIONE..... RV3D CONTIENEN GIA' LA DIFFERENZA DI UR DA ASSIMILARE TRA ANL E BCKG
            tmp = qvsat_r*rv_3d(i,k,j)/100.   !RH*qvsat+AA*qvsat*tanh(BB*gridlight(i,j))*(1.-tanh(CC*angle))
            
            total_vapor = total_vapor + Min(1.,nudge_coeff*dt)*(tmp-qv(i,k,j))*dv
            if(tmp .lt. 0.) then
            print*,'tmp minore di zero',rv_3d(i,k,j),Min(1.,nudge_coeff*dt)*tmp
!            stop
            endif
            if(tmp .gt. 0.) then
!            print*,tmp,qv(i,k,j),Min(1.,nudge_coeff*dt)*tmp,dv 
            qv(i,k,j) = qv(i,k,j)+Min(1.,nudge_coeff*dt)*tmp   !(tmp -qv(i,k,j))
            endif
            if(qv(i,k,j) .lt. 0.) qv(i,k,j)=1.e-10
!            endif
!          endif
          endif
          endif

              enddo
            enddo
          enddo


#ifdef DM_PARALLEL
      CALL MPI_Reduce(total_vapor, total_vapor_out, 1, MPI_DOUBLE_PRECISION, MPI_SUM, 0, local_communicator, mpi_error_code)

      
      
      IF ( mytask == 0 ) THEN
         total_vapor = total_vapor_out
#endif
       write(0,*) 'Total vapor mass added: ',total_vapor
       
#ifdef DM_PARALLEL
       ENDIF
#endif
       
       
        RETURN
        END SUBROUTINE r3dvar 

!+---+-----------------------------------------------------------------+
! THIS FUNCTION CALCULATES THE LIQUID SATURATION VAPOR MIXING RATIO AS
! A FUNCTION OF TEMPERATURE AND PRESSURE
!
      REAL FUNCTION RSLF(P,T)

      IMPLICIT NONE
      REAL, INTENT(IN):: P, T
      REAL:: ESL,X
      REAL, PARAMETER:: C0= .611583699E03
      REAL, PARAMETER:: C1= .444606896E02
      REAL, PARAMETER:: C2= .143177157E01
      REAL, PARAMETER:: C3= .264224321E-1
      REAL, PARAMETER:: C4= .299291081E-3
      REAL, PARAMETER:: C5= .203154182E-5
      REAL, PARAMETER:: C6= .702620698E-8
      REAL, PARAMETER:: C7= .379534310E-11
      REAL, PARAMETER:: C8=-.321582393E-13

      X=MAX(-80.,T-273.16)

!      ESL=612.2*EXP(17.67*X/(T-29.65))
      ESL=C0+X*(C1+X*(C2+X*(C3+X*(C4+X*(C5+X*(C6+X*(C7+X*C8)))))))
      RSLF=.622*ESL/(P-ESL)

      END FUNCTION RSLF 
!

!+---+-----------------------------------------------------------------+
! THIS FUNCTION CALCULATES THE ICE SATURATION VAPOR MIXING RATIO AS A
! FUNCTION OF TEMPERATURE AND PRESSURE
!
      REAL FUNCTION RSIF(P,T)

      IMPLICIT NONE 
      REAL, INTENT(IN):: P, T 
      REAL:: ESI,X
      REAL, PARAMETER:: C0= .609868993E03
      REAL, PARAMETER:: C1= .499320233E02
      REAL, PARAMETER:: C2= .184672631E01
      REAL, PARAMETER:: C3= .402737184E-1
      REAL, PARAMETER:: C4= .565392987E-3
      REAL, PARAMETER:: C5= .521693933E-5
      REAL, PARAMETER:: C6= .307839583E-7
      REAL, PARAMETER:: C7= .105785160E-9
      REAL, PARAMETER:: C8= .161444444E-12

      X=MAX(-80.,T-273.16)
      ESI=C0+X*(C1+X*(C2+X*(C3+X*(C4+X*(C5+X*(C6+X*(C7+X*C8)))))))
      RSIF=.622*ESI/(P-ESI)

      END FUNCTION RSIF 


        END MODULE module_mp_nudge_light
